"""Callbacks"""
import cPickle
import types

import BApplication
import BMessage

import jabber.jabber as jabber

from modules.constants import *
from modules.log import log
from modules.util import *

import canned

def Iq(con, iq):
	"""IQ callback"""
	query_ns = iq.getQuery()
	query_type = iq.getType()
	if (query_ns == jabber.NS_VERSION and query_type == 'get'):
		con.send(canned.versionIq(iq))
	elif (query_ns == jabber.NS_TIME and query_type == 'get'):
		con.send(canned.timeIq(iq))
	# Roster is handled by rosterlistener
	elif query_ns == jabber.NS_ROSTER:
		pass
	else :
		log('Unknown Iq: %s %s' % (query_ns, query_type))
		con.send(canned.errorIq(iq, 503,'service-unavailable'))

def Presence(con, prs):
	"""Presence callback"""
	prstype = prs.getType() 
	if prstype == 'subscribe':
		bmsg = BMessage.BMessage(INBOUND_PRESENCE)
		bmsg.AddString('type', prstype)
		jid = prs.getFrom()
		bmsg.AddString('from', toUTF8(jid.getStripped()))
		bmsg.AddString('resource', toUTF8(jid.getResource()))
		BApplication.be_app.PostMessage(bmsg)

def Message(con, msg):
	"""Mesasge callback"""
	bmsg = BMessage.BMessage(INBOUND_MESSAGE)
	jid = msg.getFrom()
	sender = jid.getStripped()
	if sender is not None:
		bmsg.AddString('from', toUTF8(sender))
		bmsg.AddString('resource', toUTF8(jid.getResource()))
	else:
		log('Message without sender!')
		return
	mtype = msg.getType()
	if mtype is None:
		mtype = 'normal'
	bmsg.AddString('type',toUTF8(mtype))
	if mtype in ['normal','chat','headline']:
		body = msg.getBody()
		if body is not None:
			bmsg.AddString('body', toUTF8(body))
		subject = msg.getSubject()
		if subject is not None:
			bmsg.AddString('subject', toUTF8(subject))
		thread = msg.getThread()
		if thread is not None:
			bmsg.AddString('thread', toUTF8(thread))
	elif mtype == 'error':
		bmsg.AddString('errorstring',toUTF8(msg.getError()))
		bmsg.AddString('errorcode', msg.getErrorCode())
	else:
		log('unsupported message type')
		return
	BApplication.be_app.PostMessage(bmsg)

def RosterListener(action, jid, info):
	"""Roster listener"""
	msg = BMessage.BMessage(INBOUND_ROSTER)
	msg.AddString('action', toUTF8(action))
	msg.AddString('jid', toUTF8(jid))
	for key in info.keys():
		value = info[key]
		if type(value) == types.ListType:
			pass
		else:
			info[key] = toUTF8(value)
	msg.AddString('info', cPickle.dumps(info))
	BApplication.be_app.PostMessage(msg)
