"""Handling incoming and outgoing messages"""
import time

import BLooper
import BFile
import BEntry
import BNode
import BNodeInfo
import BMessage
import BMessenger
import BRoster

from StorageKit import *
from SupportKit import *

from modules.constants import *
from modules.log import log

JABBER_MESSAGE_DIR = "/boot/home/jabber/In"

class Worker(BLooper.BLooper):
	def __init__(self):
		BLooper.BLooper.__init__(self, 'Message Worker')
		self.Run()
		self.msgcounter = 0

	def QuitRequested(self):
		log('Message Worker quit')
		return True

	def MessageReceived(self, msg):
		if msg.what == INBOUND_MESSAGE:
			sender = msg.FindString('from')
			mtype = msg.FindString('type')
			if mtype == 'chat':
				if not BRoster.be_roster.IsRunning(SIG_JABBERCHAT):
					BRoster.be_roster.Launch(SIG_JABBERCHAT, msg)
				else:
					messenger = BMessenger.BMessenger(SIG_JABBERCHAT)
					messenger.SendMessage(msg)
			else:
				try:
					subject = msg.FindString('subject')
				except BMessage.error:
					subject = None
				try:
					thread = msg.FindString('thread')
				except BMessage.error:
					thread = None
				try:
					body = msg.FindString('body')
				except BMessage.error:
					body = None
				# create mesage file in filesystem
				#unique = '%s-%s-%s-%i' % (sender, subject, str(time.time()), self.msgcounter)
				unique = '%s-%i' % (str(time.time()), self.msgcounter)
				bfile = BFile.BFile()
				bfile.InitCheck()
				bentry = BEntry.BEntry('%s/%s' % (JABBER_MESSAGE_DIR, unique))
				bentry.InitCheck()
				bfile.SetTo(bentry,B_CREATE_FILE|B_READ_WRITE)
				bnode = BNode.BNode(bentry)
				bnode.InitCheck()
				bnodeinfo = BNodeInfo.BNodeInfo(bnode)
				bnodeinfo.InitCheck()
				bnodeinfo.SetType('text/x-jabbermsg')
				bnode.WriteAttr('Jabber:From',B_STRING_TYPE, 0,'%s\0' % sender)
				bnode.WriteAttr('Jabber:MessageType',B_STRING_TYPE, 0,'%s\0' % mtype)
				bnode.WriteAttr('Jabber:MessageStatus',B_STRING_TYPE, 0,'unread\0')
				if subject is not None:
					bnode.WriteAttr('Jabber:Subject',B_STRING_TYPE, 0,'%s\0' % subject)
				if thread is not None:
					bnode.WriteAttr('Jabber:Thread',B_STRING_TYPE, 0,'%s\0' % thread)
				if body is not None:
					bfile.Write(body)
				self.msgcounter += 1
