""" Main """

# Application Kit
import BApplication
import BMessage
import BMessenger
# Interface Kit
import BAlert
import BBox
import BTextControl
import BTextView
import BStringView
import BButton
import BWindow
import BScrollView
from BFont import be_bold_font, be_plain_font
# Storage Kit
import BEntry
import BNode

from AppKit import *
from InterfaceKit import *
from StorageKit import *
from SupportKit import *

from modules.constants import *
import modules.ui

INPUT_CHANGED = 600

class ChatWindow(BWindow.BWindow):
	def __init__(self, jid, entry=None):
		BWindow.BWindow.__init__(self, (50,50,350,200), '', B_TITLED_WINDOW, B_NOT_ZOOMABLE|B_ASYNCHRONOUS_CONTROLS)
		bounds = self.Bounds()
		self.JID = jid
		self.bentry = entry
		# nice grey background
		box = BBox.BBox((0,0,bounds[2],bounds[3]), 'boundingbox', B_FOLLOW_ALL, B_WILL_DRAW, B_NO_BORDER)
		self.AddChild(box)
		boxbounds = box.Bounds()
		output = OutputView((5,5,100,100))
		#box.AddChild(self.output)
		input = InputView((0,0,0,0))
		box.AddChild(input)
		# set up the input
		(w,h) = input.GetPreferredSize()
		input.MoveTo(0,boxbounds[3] - h - 2)
		input.ResizeTo(boxbounds[2]-5, h)
		input.SetResizingMode(B_FOLLOW_LEFT_RIGHT|B_FOLLOW_BOTTOM)
		input.MakeFocus(True)
		iframe = input.Frame()
		# set up the output
		#view = self.FindView('output')
		output.ResizeTo(boxbounds[2]-10 - B_V_SCROLL_BAR_WIDTH, iframe[1]-10)
		output.SetTextRect(output.Bounds())
		output.SetResizingMode(B_FOLLOW_ALL)
		chatscroll = BScrollView.BScrollView('msgscroll', output, B_FOLLOW_ALL, B_WILL_DRAW, False, True, B_PLAIN_BORDER)
		box.AddChild(chatscroll)
		# Window title
		bnode = BNode.BNode(entry)
		attrinfo = bnode.GetAttrInfo('META:name')
		contact = bnode.ReadAttr('META:name',0,0,attrinfo[1]-1)
		self.SetTitle(contact)

	def MessageReceived(self, msg):
		if msg.what == INBOUND_MESSAGE:
			view = self.FindView('output')
			view.InsertText(msg.FindString('body'))
		elif msg.what == INPUT_CHANGED:
			input = self.FindView('input')
			output = self.FindView('output')
			bmsg = BMessage.BMessage(OUTBOUND_MESSAGE)
			bmsg.AddString('type', 'chat')
			bmsg.AddString('body', input.Text())
			bmsg.AddString('to', self.JID)
			try:
				messenger = BMessenger.BMessenger(SIG_HERMOD)
			except BMessenger.error:
				modules.ui.ErrorBox('Failed to send chat message. Hermod not running.', True)
			else:
				reply = messenger.SendMessageReply(bmsg)
			if reply.what != STATUS_OK:
				modules.ui.ErrorBox(reply.FindString('errstr'), True)
			else:
				output.InsertText(input.Text(), True)
			input.SetText('')


class OutputView(BTextView.BTextView):
	def __init__(self, rect):
		textrect = (1, 1, rect[2] - 1, rect[3] - 1)
		BTextView.BTextView.__init__(self, rect, 'output', textrect, B_FOLLOW_NONE,B_WILL_DRAW)
		self.MakeEditable(False)
		self.SetStylable(True)

	def InsertText(self, text, me=False):
		if me:
			color = (0, 0, 0, 0)
			prefix = '>'
		else:
			color = (150, 0, 0, 0)
			prefix = '*'
		# dont add newline if already present
		if text[-1] == '\n':
			LF = ''
		else:
			LF = '\n'
		self.Select(self.TextLength(), self.TextLength())
		self.Insert('%s %s%s' % (prefix, text, LF), [(0, be_plain_font, color)])
		self.ScrollToOffset(self.TextLength())

	def FrameResized(self, width, height):
		self.SetTextRect((1,1,width - 1, height - 1))


class InputView(BTextControl.BTextControl):
	def __init__(self, rect):
		bmsg = BMessage.BMessage(INPUT_CHANGED)
		BTextControl.BTextControl.__init__(self, rect,'input', None ,'',bmsg, B_FOLLOW_NONE, B_WILL_DRAW)
