""" Main """

# Application Kit
import BApplication
import BMessage
import BMessenger
# Interface Kit
import BAlert
import BBox
import BTextControl
import BStringView
import BButton
import BWindow
# Storage Kit
import BEntry
import BNode

from AppKit import *
from InterfaceKit import *
from StorageKit import *
from SupportKit import *

from modules.constants import *
import modules.ui

#
SUBSCRIBE_REQUEST		= 600
UNSUBSCRIBE_REQUEST		= 601
UPDATE_STATUS			= 602

class ContactWindow(BWindow.BWindow):
	def __init__(self, entry):
		BWindow.BWindow.__init__(self, (50,50,350,200), '', 
			B_TITLED_WINDOW, B_NOT_ZOOMABLE|B_ASYNCHRONOUS_CONTROLS)
		self.bentry = entry
		bounds = self.Bounds()
		# nice grey background
		box = BBox.BBox((0,0,bounds[2],bounds[3]), 'boundingbox', B_FOLLOW_ALL, B_WILL_DRAW, B_NO_BORDER)
		self.AddChild(box)
		# jid control
		jidctl = BTextControl.BTextControl((1,1,bounds[2]-2,0), 'jidctl', 'Jabber ID', '', None, B_FOLLOW_LEFT_RIGHT, B_WILL_DRAW|B_FULL_UPDATE_ON_RESIZE|B_NAVIGABLE)
		jid = ''
		if entry is not None:
			bnode = BNode.BNode(entry)
			bnode.InitCheck()
			try:
				attrinfo = bnode.GetAttrInfo('Jabber:JID')
				jid = bnode.ReadAttr('Jabber:JID',0,0,attrinfo[1]-1)
			except BNode.error:
				pass
			else:
				jidctl.SetText(jid)
			attrinfo = bnode.GetAttrInfo('META:name')
			contact = bnode.ReadAttr('META:name',0,0,attrinfo[1]-1)
			self.SetTitle(contact)
		(x,y) = jidctl.GetPreferredSize()
		jidctl.ResizeTo(bounds[2]-2,y)
		div = jidctl.StringWidth('Jabber ID')
		jidctl.SetDivider(div+5)
		box.AddChild(jidctl)
		wframe = jidctl.Frame()
		statbox = BBox.BBox((5,wframe[3]+15,bounds[2]-5,bounds[3]-5), 'statusbox', B_FOLLOW_NONE, B_WILL_DRAW|B_FULL_UPDATE_ON_RESIZE, B_FANCY_BORDER)
		statbox.SetLabel('Presence')
		box.AddChild(statbox)
		sbounds = statbox.Bounds()
		strlist = [('onlinelabel','Online'),('showlabel','Show'),('statuslabel','Status'),('subscriptionlabel','Subscription')]
		y = 20
		lw = 0
		for (name, label) in strlist:
			stringview = BStringView.BStringView((10,y,0,0),name,label)
			stringview.ResizeToPreferred()
			(x,y,w,h) = stringview.Frame()
			if w > lw:
				lw = w
			y = h + 5
			statbox.AddChild(stringview)
		strlist = [('online','n/a'),('show','n/a'),('status','n/a'),('subscription','n/a')]
		y = 20
		for (name, label) in strlist:
			stringview = BStringView.BStringView((20 + lw + 10,y,0,0),name,label)
			stringview.ResizeToPreferred()
			(x,y,w,h) = stringview.Frame()
			y = h + 5
			statbox.AddChild(stringview)
		statframe = statbox.Frame()
		statbox.ResizeTo(statframe[2],y+5)
		statframe = statbox.Frame()
		btnlist = [('subscribebtn','Subscribe', BMessage.BMessage(SUBSCRIBE_REQUEST)),
			('unsubscribebtn','Unsubscribe', BMessage.BMessage(UNSUBSCRIBE_REQUEST)),
			('updatebtn','Update Status', BMessage.BMessage(UPDATE_STATUS))]
		y = statframe[3] + 5
		x = 5
		for (name, label, msg) in btnlist:
			btn = BButton.BButton((x,y,0,0),name,label, msg, B_FOLLOW_NONE, B_WILL_DRAW|B_NAVIGABLE)
			btn.ResizeToPreferred()
			(x,y,w,h) = btn.Frame()
			box.AddChild(btn)
			x = w + 5
		if statframe[2] > w :
			self.ResizeTo(statframe[2] + 5, h + 5)
		else:
			self.ResizeTo(w + 5, h + 5)
		for (name, label,msg) in btnlist:
			btn = self.FindView(name)
			btn.SetResizingMode(B_FOLLOW_BOTTOM)
		statbox.SetResizingMode(B_FOLLOW_ALL)
		# save jid
		self.origjid = jid
		self.JID = jid
		# update status form Hermod
		self._requestRosterItem()
		jidctl.MakeFocus(True)

	def QuitRequested(self):
		view = self.FindView('jidctl')
		newjid = view.Text()
		if newjid.strip() == '' and self.origjid != '':
			dlg = BAlert.BAlert('Remove JID Dialog', 'Remove Jabber ID from %s?' % self.Title(), 'Sure thing, Sonny', 'Are you mad?', None, None, B_WARNING_ALERT)
			if dlg.Go() == 0:
				bnode = BNode.BNode(self.bentry)
				bnode.InitCheck()
				try:
					bnode.RemoveAttr('Jabber:JID')
				except BNode.error:
					dlg = BAlert.BAlert('Remove Error Dialog', 'There was an error removing the Jabber ID.', 'That sucks', None, None, None, B_STOP_ALERT)
					dlg.Go(None)
		elif self.origjid != newjid:
			dlg = BAlert.BAlert('Save Dialog', 'Save changes to %s?' % self.Title(), 'Sure thing, Sonny', 'Are you mad?', None, None, B_WARNING_ALERT)
			if dlg.Go() == 0:
				bnode = BNode.BNode(self.bentry)
				bnode.InitCheck()
				try:
					bnode.WriteAttr('Jabber:JID',B_STRING_TYPE,0,'%s\0' % view.Text())
				except BNode.error:
					dlg = BAlert.BAlert('Save Error Dialog', 'There was an error writing the changes.', 'That sucks', None, None, None, B_STOP_ALERT)
					dlg.Go(None)
		return True

	def MessageReceived(self, msg):
		if msg.what == UPDATE_STATUS:
			self._requestRosterItem()
		elif msg.what == SUBSCRIBE_REQUEST:
			try:
				messenger = BMessenger.BMessenger(SIG_HERMOD)
			except BMessenger.error:
				modules.ui.ErrorBox('Hermod not running,', True)
			else:
				bmsg = BMessage.BMessage(OUTBOUND_PRESENCE)
				bmsg.AddString('to',self.JID)
				bmsg.AddString('type', 'subscribe')
				reply = messenger.SendMessageReply(bmsg)
				if reply.what == STATUS_ERROR:
					modules.ui.ErrorBox(reply.FindString('errstr'), True)
		elif msg.what == UNSUBSCRIBE_REQUEST:
			try:
				messenger = BMessenger.BMessenger(SIG_HERMOD)
			except BMessenger.error:
				modules.ui.ErrorBox('Hermod not running,', True)
			else:
				bmsg = BMessage.BMessage(OUTBOUND_PRESENCE)
				bmsg.AddString('to',self.JID)
				bmsg.AddString('type', 'unsubscribe')
				messenger.SendMessage(bmsg)
				bmsg = BMessage.BMessage(OUTBOUND_PRESENCE)
				bmsg.AddString('to',self.JID)
				bmsg.AddString('type', 'unsubscribed')
				reply = messenger.SendMessageReply(bmsg)
				if reply.what == STATUS_ERROR:
					modules.ui.ErrorBox(reply.FindString('errstr'), True)
		elif msg.what == 1145132097:
			# we got a drag'n'drop even, probably from Tracker
			drop_point = msg.FindPoint('_drop_point_')
			try:
				eref = msg.FindRef('refs')
			except BMessage.error:
				# no refs in drop message
				return
			realpoint = self.ConvertFromScreen(drop_point)
			view = self.FindView(realpoint)
			# this is not really a rock-solid way to find the right control...
			if view.Name() == '_input_':
				txc = view.Parent()
				bnode = BNode.BNode(eref)
				try:
					bnode.InitCheck()
				except BNode.error:
					# Oooops, shouldnt happen. Lets just silently ignore it....
					return
				try:
					attrinfo = bnode.GetAttrInfo('Jabber:JID')
					jid = bnode.ReadAttr('Jabber:JID',0,0,attrinfo[1]-1)
					txc.SetText(jid)
				except BNode.error:
					# no jid in Person file
					pass

	###
	def _requestRosterItem(self):
		ctls = [('subscription','sub'),('online','online'),('show','show'),('status','status')]
		bmsg = BMessage.BMessage(ROSTER_ITEM_REQUEST)
		bmsg.AddString('jid', self.JID)
		try:
			messenger = BMessenger.BMessenger(SIG_HERMOD)
		except BMessenger.error:
			modules.ui.ErrorBox('Hermod not running,', True)
		else:
			reply = messenger.SendMessageReply(bmsg)
			if reply.what == STATUS_ERROR:
				reply.PrintToStream()
				modules.ui.ErrorBox(reply.FindString('errstr'), True)
				for (name, msgitem) in ctls:
					view = self.FindView(name)
					view.SetText('n/a')
					view.ResizeToPreferred()
			else:
				for (name, msgitem) in ctls:
					view = self.FindView(name)
					try:
						view.SetText(reply.FindString(msgitem))
						view.ResizeToPreferred()
					except BMessage.error:
						view.SetText('-')
