"""Base for preference pages"""

import BBox
import BMenuField
import BTextControl
import BCheckBox
from InterfaceKit import *

from modules.constants import *
from constants import *
from widgets import *

class PrefsBox(BBox.BBox):
	"""Base class for preference pages"""
	def __init__(self, rect, name, border=B_FANCY_BORDER):
		BBox.BBox.__init__(self, rect, name, B_FOLLOW_ALL, B_FULL_UPDATE_ON_RESIZE|B_WILL_DRAW, border)
		self.changed = False
		self.savecontrols = None
		self.conf = None
		self.section = None

	def hasChanged(self):
		"""Returns True of any changes bas been made to the page"""
		return self.changed

	def saveSettings(self):
		"""Saves settings"""
		if self.savecontrols is not None:
			if self.changed:
				if not showSavePrompt():
					return
				for ctl in self.savecontrols:
					c = self.FindView(ctl)
					if c.IsEnabled():
						if isinstance(c,BTextControl.BTextControl):
							value = c.Text()
							self.conf.set(self.section,ctl,value)
						elif isinstance(c,BCheckBox.BCheckBox):
							if c.Value() == False:
								self.conf.set(self.section,ctl,'no')
							else:
								self.conf.set(self.section,ctl,'yes')
						elif isinstance(c,BMenuField.BMenuField):
							menu = c.Menu()
							item = menu.FindMarked()
							self.conf.set(self.section,ctl,item.Label())
					else:
						self.conf.remove_option(self.section,ctl)
				self.conf.write()

	def restoreSettings(self, custom=None):
		"""Restore settings"""
		if self.section is not None and self.savecontrols is not None:
			options = self.conf.options(self.section)
			print options
			for option in options:
				value = self.conf.get(self.section,option)
				v = self.FindView(option)
				if isinstance(v,BMenuField.BMenuField):
					menu = v.Menu()
					try:
						item = menu.FindItem(value)
					except ValueError:
						pass
					else:
						item.SetMarked(True)

	def controlEvent(self, msg):
		"""Stub function to handle control events. Default implementation simply sets the change-state to True"""
		self.changed = True
