"""Preferences Window"""

# Application Kit
import BApplication
import BMessage
# Interface Kit
import BAlert
import BBox
import BWindow
import BListView
import BStringItem
import BPopUpMenu
import BMenuField
import BMenuItem
# Symbols
from AppKit import *
from InterfaceKit import *

from modules.constants import *
from constants import *

from widgets import BoundingBox
from pages.LoggingPrefs import LoggingBox
from pages.GeneralPrefs import GeneralBox
from pages.PresencePrefs import PresenceBox
from pages.AccountsPrefs import AccountsBox

class PrefsWindow(BWindow.BWindow):
	def __init__(self):
		BWindow.BWindow.__init__(self, (50,50,400,300), 'Hermod Preferences', B_TITLED_WINDOW, B_QUIT_ON_WINDOW_CLOSE|B_NOT_ZOOMABLE|B_ASYNCHRONOUS_CONTROLS)
		bounds = self.Bounds()
		# nice grey background
		box = BoundingBox((0,0,bounds[2],bounds[3]))
		self.AddChild(box)
		boxbounds = box.Bounds()
		lv = BListView.BListView((2,2,100,boxbounds[3]-2),'listview',B_SINGLE_SELECTION_LIST,B_FOLLOW_TOP_BOTTOM,B_WILL_DRAW)
		lv.AddItem(BStringItem.BStringItem('General'))
		lv.AddItem(BStringItem.BStringItem('Logging'))
		lv.AddItem(BStringItem.BStringItem('Presence'))
		lv.AddItem(BStringItem.BStringItem('Accounts'))
		bmsg = BMessage.BMessage(SELECTION_CHANGED)
		lv.SetSelectionMessage(bmsg)
		box.AddChild(lv)
		lv.ResizeToPreferred()
		lvframe = lv.Frame()
		self.prefX = lvframe[2] + 10
		self.prefY = 5
		self.activepref = None

	def MessageReceived(self, msg):
		if msg.what == SELECTION_CHANGED:
			lv = self.FindView('listview')
			ix = lv.CurrentSelection()
			box = self.FindView('boundingbox')
			if ix > -1:
				if self.activepref is not None:
					self.activepref.saveSettings()
					box.RemoveChild(self.activepref)
				li = lv.ItemAt(ix)
				bounds = box.Bounds()
				if li.Text() == 'General':
					self.activepref = GeneralBox((self.prefX,self.prefY,bounds[2] - 5, bounds[3] - 5))
				elif li.Text() == 'Presence':
					self.activepref = PresenceBox((self.prefX,self.prefY,bounds[2] - 5, bounds[3] - 5))
				elif li.Text() == 'Logging':
					self.activepref = LoggingBox((self.prefX,self.prefY,bounds[2] - 5, bounds[3] - 5))
				elif li.Text() == 'Accounts':
					self.activepref = AccountsBox((self.prefX,self.prefY,bounds[2] - 5, bounds[3] - 5))
				box.AddChild(self.activepref)
				self.activepref.restoreSettings()
			else:
				box.RemoveChild(self.activepref)
				self.activepref = None
		elif msg.what == CONTROL_EVENT:
			self.activepref.controlEvent(msg)

	def QuitRequested(self):
		if self.activepref is not None:
			self.activepref.saveSettings()
		return True
